@@ -129,23 +129,24 @@ void markPredictionBlockBoundary(de265_image* img, int x0,int y0,
 
 bool derive_edgeFlags_CTBRow(de265_image* img, int ctby)
 {
-  const int minCbSize = img->sps.MinCbSizeY;
+  const seq_parameter_set& sps = img->get_sps();
+  const pic_parameter_set& pps = img->get_pps();
+
+  const int minCbSize = sps.MinCbSizeY;
   bool deblocking_enabled=false; // whether deblocking is enabled in some part of the image
 
-  int ctb_mask = (1<<img->sps.Log2CtbSizeY)-1;
-  int picWidthInCtbs = img->sps.PicWidthInCtbsY;
-  int ctbshift = img->sps.Log2CtbSizeY;
-
-  const pic_parameter_set* pps = &img->pps;
+  int ctb_mask = (1<<sps.Log2CtbSizeY)-1;
+  int picWidthInCtbs = sps.PicWidthInCtbsY;
+  int ctbshift = sps.Log2CtbSizeY;
 
 
-  int cb_y_start = ( ctby    << img->sps.Log2CtbSizeY) >> img->sps.Log2MinCbSizeY;
-  int cb_y_end   = ((ctby+1) << img->sps.Log2CtbSizeY) >> img->sps.Log2MinCbSizeY;
+  int cb_y_start = ( ctby    << sps.Log2CtbSizeY) >> sps.Log2MinCbSizeY;
+  int cb_y_end   = ((ctby+1) << sps.Log2CtbSizeY) >> sps.Log2MinCbSizeY;
 
-  cb_y_end = std::min(cb_y_end, img->sps.PicHeightInMinCbsY);
+  cb_y_end = std::min(cb_y_end, sps.PicHeightInMinCbsY);
 
   for (int cb_y=cb_y_start;cb_y<cb_y_end;cb_y++)
-    for (int cb_x=0;cb_x<img->sps.PicWidthInMinCbsY;cb_x++)
+    for (int cb_x=0;cb_x<img->get_sps().PicWidthInMinCbsY;cb_x++)
       {
         int log2CbSize = img->get_log2CbSize_cbUnits(cb_x,cb_y);
         if (log2CbSize==0) {
@@ -185,9 +186,9 @@ bool derive_edgeFlags_CTBRow(de265_image* img, int ctby)
             {
               filterLeftCbEdge = 0;
             }
-          else if (pps->loop_filter_across_tiles_enabled_flag == 0 &&
-                   pps->TileIdRS[  x0ctb           +y0ctb*picWidthInCtbs] !=
-                   pps->TileIdRS[((x0-1)>>ctbshift)+y0ctb*picWidthInCtbs]) {
+          else if (pps.loop_filter_across_tiles_enabled_flag == 0 &&
+                   pps.TileIdRS[  x0ctb           +y0ctb*picWidthInCtbs] !=
+                   pps.TileIdRS[((x0-1)>>ctbshift)+y0ctb*picWidthInCtbs]) {
             filterLeftCbEdge = 0;
           }
         }
@@ -199,9 +200,9 @@ bool derive_edgeFlags_CTBRow(de265_image* img, int ctby)
             {
               filterTopCbEdge = 0;
             }
-          else if (pps->loop_filter_across_tiles_enabled_flag == 0 &&
-                   pps->TileIdRS[x0ctb+  y0ctb           *picWidthInCtbs] !=
-                   pps->TileIdRS[x0ctb+((y0-1)>>ctbshift)*picWidthInCtbs]) {
+          else if (pps.loop_filter_across_tiles_enabled_flag == 0 &&
+                   pps.TileIdRS[x0ctb+  y0ctb           *picWidthInCtbs] !=
+                   pps.TileIdRS[x0ctb+((y0-1)>>ctbshift)*picWidthInCtbs]) {
             filterTopCbEdge = 0;
           }
         }
@@ -228,7 +229,7 @@ bool derive_edgeFlags(de265_image* img)
 {
   bool deblocking_enabled=false;
 
-  for (int y=0;y<img->sps.PicHeightInCtbsY;y++) {
+  for (int y=0;y<img->get_sps().PicHeightInCtbsY;y++) {
     deblocking_enabled |= derive_edgeFlags_CTBRow(img,y);
   }
 
@@ -252,8 +253,8 @@ void derive_boundaryStrength(de265_image* img, bool vertical, int yStart,int yEn
   xEnd = libde265_min(xEnd,img->get_deblk_width());
   yEnd = libde265_min(yEnd,img->get_deblk_height());
 
-  int TUShift = img->sps.Log2MinTrafoSize;
-  int TUStride= img->sps.PicWidthInTbsY;
+  int TUShift = img->get_sps().Log2MinTrafoSize;
+  int TUStride= img->get_sps().PicWidthInTbsY;
 
   for (int y=yStart;y<yEnd;y+=yIncr)
     for (int x=xStart;x<xEnd;x+=xIncr) {
@@ -288,16 +289,16 @@ void derive_boundaryStrength(de265_image* img, bool vertical, int yStart,int yEn
 
             bS = 0;
 
-            const MotionVectorSpec* mviP = img->get_mv_info(xDiOpp,yDiOpp);
-            const MotionVectorSpec* mviQ = img->get_mv_info(xDi   ,yDi);
+            const PBMotion& mviP = img->get_mv_info(xDiOpp,yDiOpp);
+            const PBMotion& mviQ = img->get_mv_info(xDi   ,yDi);
 
             slice_segment_header* shdrP = img->get_SliceHeader(xDiOpp,yDiOpp);
             slice_segment_header* shdrQ = img->get_SliceHeader(xDi   ,yDi);
 
-            int refPicP0 = mviP->predFlag[0] ? shdrP->RefPicList[0][ mviP->refIdx[0] ] : -1;
-            int refPicP1 = mviP->predFlag[1] ? shdrP->RefPicList[1][ mviP->refIdx[1] ] : -1;
-            int refPicQ0 = mviQ->predFlag[0] ? shdrQ->RefPicList[0][ mviQ->refIdx[0] ] : -1;
-            int refPicQ1 = mviQ->predFlag[1] ? shdrQ->RefPicList[1][ mviQ->refIdx[1] ] : -1;
+            int refPicP0 = mviP.predFlag[0] ? shdrP->RefPicList[0][ mviP.refIdx[0] ] : -1;
+            int refPicP1 = mviP.predFlag[1] ? shdrP->RefPicList[1][ mviP.refIdx[1] ] : -1;
+            int refPicQ0 = mviQ.predFlag[0] ? shdrQ->RefPicList[0][ mviQ.refIdx[0] ] : -1;
+            int refPicQ1 = mviQ.predFlag[1] ? shdrQ->RefPicList[1][ mviQ.refIdx[1] ] : -1;
 
             bool samePics = ((refPicP0==refPicQ0 && refPicP1==refPicQ1) ||
                              (refPicP0==refPicQ1 && refPicP1==refPicQ0));
@@ -306,13 +307,13 @@ void derive_boundaryStrength(de265_image* img, bool vertical, int yStart,int yEn
               bS = 1;
             }
             else {
-              MotionVector mvP0 = mviP->mv[0]; if (!mviP->predFlag[0]) { mvP0.x=mvP0.y=0; }
-              MotionVector mvP1 = mviP->mv[1]; if (!mviP->predFlag[1]) { mvP1.x=mvP1.y=0; }
-              MotionVector mvQ0 = mviQ->mv[0]; if (!mviQ->predFlag[0]) { mvQ0.x=mvQ0.y=0; }
-              MotionVector mvQ1 = mviQ->mv[1]; if (!mviQ->predFlag[1]) { mvQ1.x=mvQ1.y=0; }
+              MotionVector mvP0 = mviP.mv[0]; if (!mviP.predFlag[0]) { mvP0.x=mvP0.y=0; }
+              MotionVector mvP1 = mviP.mv[1]; if (!mviP.predFlag[1]) { mvP1.x=mvP1.y=0; }
+              MotionVector mvQ0 = mviQ.mv[0]; if (!mviQ.predFlag[0]) { mvQ0.x=mvQ0.y=0; }
+              MotionVector mvQ1 = mviQ.mv[1]; if (!mviQ.predFlag[1]) { mvQ1.x=mvQ1.y=0; }
 
-              int numMV_P = mviP->predFlag[0] + mviP->predFlag[1];
-              int numMV_Q = mviQ->predFlag[0] + mviQ->predFlag[1];
+              int numMV_P = mviP.predFlag[0] + mviP.predFlag[1];
+              int numMV_Q = mviQ.predFlag[0] + mviQ.predFlag[1];
 
               if (numMV_P!=numMV_Q) {
                 img->decctx->add_warning(DE265_WARNING_NUMMVP_NOT_EQUAL_TO_NUMMVQ, false);
@@ -376,7 +377,7 @@ void derive_boundaryStrength(de265_image* img, bool vertical, int yStart,int yEn
 
 void derive_boundaryStrength_CTB(de265_image* img, bool vertical, int xCtb,int yCtb)
 {
-  int ctbSize = img->sps.CtbSizeY;
+  int ctbSize = img->get_sps().CtbSizeY;
   int deblkSize = ctbSize/4;
 
   derive_boundaryStrength(img,vertical,
@@ -406,12 +407,14 @@ void edge_filtering_luma_internal(de265_image* img, bool vertical,
 {
   //printf("luma %d-%d %d-%d\n",xStart,xEnd,yStart,yEnd);
 
+  const seq_parameter_set& sps = img->get_sps();
+
   int xIncr = vertical ? 2 : 1;
   int yIncr = vertical ? 1 : 2;
 
   const int stride = img->get_image_stride(0);
 
-  int bitDepth_Y = img->sps.BitDepth_Y;
+  int bitDepth_Y = sps.BitDepth_Y;
 
   xEnd = libde265_min(xEnd,img->get_deblk_width());
   yEnd = libde265_min(yEnd,img->get_deblk_height());
@@ -572,17 +575,17 @@ void edge_filtering_luma_internal(de265_image* img, bool vertical,
           bool filterQ = true;
 
           if (vertical) {
-            if (img->sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(xDi-1,yDi)) filterP=false;
+            if (sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(xDi-1,yDi)) filterP=false;
             if (img->get_cu_transquant_bypass(xDi-1,yDi)) filterP=false;
 
-            if (img->sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(xDi,yDi)) filterQ=false;
+            if (sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(xDi,yDi)) filterQ=false;
             if (img->get_cu_transquant_bypass(xDi,yDi)) filterQ=false;
           }
           else {
-            if (img->sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(xDi,yDi-1)) filterP=false;
+            if (sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(xDi,yDi-1)) filterP=false;
             if (img->get_cu_transquant_bypass(xDi,yDi-1)) filterP=false;
 
-            if (img->sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(xDi,yDi)) filterQ=false;
+            if (sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(xDi,yDi)) filterQ=false;
             if (img->get_cu_transquant_bypass(xDi,yDi)) filterQ=false;
           }
 
@@ -709,7 +712,7 @@ void edge_filtering_luma(de265_image* img, bool vertical,
 
 void edge_filtering_luma_CTB(de265_image* img, bool vertical, int xCtb,int yCtb)
 {
-  int ctbSize = img->sps.CtbSizeY;
+  int ctbSize = img->get_sps().CtbSizeY;
   int deblkSize = ctbSize/4;
 
   edge_filtering_luma(img,vertical,
@@ -730,8 +733,10 @@ void edge_filtering_chroma_internal(de265_image* img, bool vertical,
 {
   //printf("chroma %d-%d %d-%d\n",xStart,xEnd,yStart,yEnd);
 
-  const int SubWidthC  = img->sps.SubWidthC;
-  const int SubHeightC = img->sps.SubHeightC;
+  const seq_parameter_set& sps = img->get_sps();
+
+  const int SubWidthC  = sps.SubWidthC;
+  const int SubHeightC = sps.SubHeightC;
 
   int xIncr = vertical ? 2 : 1;
   int yIncr = vertical ? 1 : 2;
@@ -744,7 +749,7 @@ void edge_filtering_chroma_internal(de265_image* img, bool vertical,
   xEnd = libde265_min(xEnd,img->get_deblk_width());
   yEnd = libde265_min(yEnd,img->get_deblk_height());
 
-  int bitDepth_C = img->sps.BitDepth_C;
+  int bitDepth_C = sps.BitDepth_C;
 
   for (int y=yStart;y<yEnd;y+=yIncr)
     for (int x=xStart;x<xEnd;x+=xIncr) {
@@ -760,8 +765,8 @@ void edge_filtering_chroma_internal(de265_image* img, bool vertical,
 
         for (int cplane=0;cplane<2;cplane++) {
           int cQpPicOffset = (cplane==0 ?
-                              img->pps.pic_cb_qp_offset :
-                              img->pps.pic_cr_qp_offset);
+                              img->get_pps().pic_cb_qp_offset :
+                              img->get_pps().pic_cr_qp_offset);
 
           pixel_t* ptr = img->get_image_plane_at_pos_NEW<pixel_t>(cplane+1, xDi,yDi);
 
@@ -807,7 +812,7 @@ void edge_filtering_chroma_internal(de265_image* img, bool vertical,
                       img->get_QPY(SubWidthC*xDi,SubHeightC*yDi-1));
           int qP_i = ((QP_Q+QP_P+1)>>1) + cQpPicOffset;
           int QP_C;
-          if (img->sps.ChromaArrayType == CHROMA_420) {
+          if (sps.ChromaArrayType == CHROMA_420) {
             QP_C = table8_22(qP_i);
           } else {
             QP_C = libde265_min(qP_i, 51);
@@ -824,17 +829,17 @@ void edge_filtering_chroma_internal(de265_image* img, bool vertical,
           int Q = Clip3(0,53, QP_C + 2*(bS-1) + tc_offset);
 
           int tcPrime = table_8_23_tc[Q];
-          int tc = tcPrime * (1<<(img->sps.BitDepth_C - 8));
+          int tc = tcPrime * (1<<(sps.BitDepth_C - 8));
 
           logtrace(LogDeblock,"tc_offset=%d Q=%d tc'=%d tc=%d\n",tc_offset,Q,tcPrime,tc);
 
           if (vertical) {
             bool filterP = true;
-            if (img->sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(SubWidthC*xDi-1,SubHeightC*yDi)) filterP=false;
+            if (sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(SubWidthC*xDi-1,SubHeightC*yDi)) filterP=false;
             if (img->get_cu_transquant_bypass(SubWidthC*xDi-1,SubHeightC*yDi)) filterP=false;
 
             bool filterQ = true;
-            if (img->sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(SubWidthC*xDi,SubHeightC*yDi)) filterQ=false;
+            if (sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(SubWidthC*xDi,SubHeightC*yDi)) filterQ=false;
             if (img->get_cu_transquant_bypass(SubWidthC*xDi,SubHeightC*yDi)) filterQ=false;
 
 
@@ -847,11 +852,11 @@ void edge_filtering_chroma_internal(de265_image* img, bool vertical,
           }
           else {
             bool filterP = true;
-            if (img->sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(SubWidthC*xDi,SubHeightC*yDi-1)) filterP=false;
+            if (sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(SubWidthC*xDi,SubHeightC*yDi-1)) filterP=false;
             if (img->get_cu_transquant_bypass(SubWidthC*xDi,SubHeightC*yDi-1)) filterP=false;
 
             bool filterQ = true;
-            if (img->sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(SubWidthC*xDi,SubHeightC*yDi)) filterQ=false;
+            if (sps.pcm_loop_filter_disable_flag && img->get_pcm_flag(SubWidthC*xDi,SubHeightC*yDi)) filterQ=false;
             if (img->get_cu_transquant_bypass(SubWidthC*xDi,SubHeightC*yDi)) filterQ=false;
 
             for (int k=0;k<4;k++) {
@@ -880,7 +885,7 @@ void edge_filtering_chroma(de265_image* img, bool vertical, int yStart,int yEnd,
 
 void edge_filtering_chroma_CTB(de265_image* img, bool vertical, int xCtb,int yCtb)
 {
-  int ctbSize = img->sps.CtbSizeY;
+  int ctbSize = img->get_sps().CtbSizeY;
   int deblkSize = ctbSize/4;
 
   edge_filtering_chroma(img,vertical,
@@ -914,7 +919,7 @@ void thread_task_deblock_CTBRow::work()
   int xStart=0;
   int xEnd = img->get_deblk_width();
 
-  int ctbSize = img->sps.CtbSizeY;
+  int ctbSize = img->get_sps().CtbSizeY;
   int deblkSize = ctbSize/4;
 
   int first =  ctb_y    * deblkSize;
@@ -926,12 +931,12 @@ void thread_task_deblock_CTBRow::work()
   int finalProgress = CTB_PROGRESS_DEBLK_V;
   if (!vertical) finalProgress = CTB_PROGRESS_DEBLK_H;
 
-  int rightCtb = img->sps.PicWidthInCtbsY-1;
+  int rightCtb = img->get_sps().PicWidthInCtbsY-1;
 
   if (vertical) {
     // pass 1: vertical
 
-    int CtbRow = std::min(ctb_y+1 , img->sps.PicHeightInCtbsY-1);
+    int CtbRow = std::min(ctb_y+1 , img->get_sps().PicHeightInCtbsY-1);
     img->wait_for_progress(this, rightCtb,CtbRow, CTB_PROGRESS_PREFILTER);
   }
   else {
@@ -943,7 +948,7 @@ void thread_task_deblock_CTBRow::work()
 
     img->wait_for_progress(this, rightCtb,ctb_y,  CTB_PROGRESS_DEBLK_V);
 
-    if (ctb_y+1<img->sps.PicHeightInCtbsY) {
+    if (ctb_y+1<img->get_sps().PicHeightInCtbsY) {
       img->wait_for_progress(this, rightCtb,ctb_y+1, CTB_PROGRESS_DEBLK_V);
     }
   }
@@ -969,13 +974,13 @@ void thread_task_deblock_CTBRow::work()
 
     edge_filtering_luma(img, vertical, first,last, xStart,xEnd);
 
-    if (img->sps.ChromaArrayType != CHROMA_MONO) {
+    if (img->get_sps().ChromaArrayType != CHROMA_MONO) {
       edge_filtering_chroma(img, vertical, first,last, xStart,xEnd);
     }
   }
 
   for (int x=0;x<=rightCtb;x++) {
-    const int CtbWidth = img->sps.PicWidthInCtbsY;
+    const int CtbWidth = img->get_sps().PicWidthInCtbsY;
     img->ctb_progress[x+ctb_y*CtbWidth].set_progress(finalProgress);
   }
 
@@ -989,14 +994,14 @@ void add_deblocking_tasks(image_unit* imgunit)
   de265_image* img = imgunit->img;
   decoder_context* ctx = img->decctx;
 
-  int nRows = img->sps.PicHeightInCtbsY;
+  int nRows = img->get_sps().PicHeightInCtbsY;
 
   int n=0;
   img->thread_start(nRows*2);
 
   for (int pass=0;pass<2;pass++)
     {
-      for (int y=0;y<img->sps.PicHeightInCtbsY;y++)
+      for (int y=0;y<img->get_sps().PicHeightInCtbsY;y++)
         {
           thread_task_deblock_CTBRow* task = new thread_task_deblock_CTBRow;
 
@@ -1026,7 +1031,7 @@ void apply_deblocking_filter(de265_image* img) // decoder_context* ctx)
       derive_boundaryStrength(img, true ,0,img->get_deblk_height(),0,img->get_deblk_width());
       edge_filtering_luma    (img, true ,0,img->get_deblk_height(),0,img->get_deblk_width());
 
-      if (img->sps.ChromaArrayType != CHROMA_MONO) {
+      if (img->get_sps().ChromaArrayType != CHROMA_MONO) {
         edge_filtering_chroma  (img, true ,0,img->get_deblk_height(),0,img->get_deblk_width());
       }
 #if 0
@@ -1041,7 +1046,7 @@ void apply_deblocking_filter(de265_image* img) // decoder_context* ctx)
       derive_boundaryStrength(img, false ,0,img->get_deblk_height(),0,img->get_deblk_width());
       edge_filtering_luma    (img, false ,0,img->get_deblk_height(),0,img->get_deblk_width());
 
-      if (img->sps.ChromaArrayType != CHROMA_MONO) {
+      if (img->get_sps().ChromaArrayType != CHROMA_MONO) {
         edge_filtering_chroma  (img, false ,0,img->get_deblk_height(),0,img->get_deblk_width());
       }
 
